from util import common_start

def build_safe_pattern(first_number, last_number):
	""" Builds a pattern that covers digits from first_number to last_number
	Assumes that this range is safe to generate a pattern for

	Will generate a DialplanException if first_number and last_number are not of equal length

	:param first_number: telephone number in E164 format
	:type first_number: integer
	:param last_number: telephoen number in E164 format
	:type last_number: integer
	:returns: string representing the pattern
	:rtype: string
	"""

	from dialplan_exceptions import DialplanException

	fn_string = str(first_number)
	ln_string = str(last_number)

	if len(fn_string) != len(ln_string):
		#the pattern to match is not valid
		raise DialplanException("First and last numbers are not of equal length")

	if int(first_number) > int(last_number):
		raise DialplanException("Last number is smaller than the first number")

	if first_number == last_number:
		return fn_string


	#lets work out the number of significant digits
	pattern = common_start(first_number,last_number)
	digit_start = int(len(pattern))
	digits = len(str(first_number)) - len(pattern)
	strip_digits = int(digits*-1)
	remaining_fn = fn_string[strip_digits:]
	remaining_ln = ln_string[strip_digits:]

	for fdigit, ldigit in zip(list(remaining_fn),list(remaining_ln)):
		if fdigit == "0" and ldigit == "9":
			pattern = pattern + "X"
		elif fdigit == "1" and ldigit == "9":
			pattern = pattern + "Z"
		elif fdigit == "2" and ldigit == "9":
			pattern = pattern + "N"
		else:
			pattern = pattern + "[" + fdigit + "-" + ldigit + "]"

	return "_"+pattern

def generate_patterns(low,high):
	""" Builds a list of patterns that cover the E.164 numbers between low and high 

	Will generate a DialplanException if first_number and last_number are not of equal length

	:param first_number: telephone number in E164 format
	:type first_number: integer
	:param last_number: telephoen number in E164 format
	:type last_number: integer
	:returns: list of strings representing the pattern
	:rtype: list

	"""

	from ranges import split_range

	from dialplan_exceptions import DialplanException

	#get the string and integer reps of the numbers
	fn_string = str(low)
	ln_string = str(high)

	low = int(low)
	high = int(high)

	if len(fn_string) != len(ln_string):
		#the pattern to match is not valid
		raise DialplanException("First and last numbers are not of equal length")

	if low > high:
		raise DialplanException("Last number is smaller than the first number")

	patterns = []

	range_list = split_range(low,high)
	for rangeTuple in range_list:
		patterns.append(build_safe_pattern(rangeTuple[0], rangeTuple[1]))
	return patterns



def common_start(first_number, last_number):
	""" Get the common starting characters of the two strings first_number and last_number """
	str_first = str(first_number)
	str_last = str(last_number)
	common = ""
	for a,b in zip(list(str_first), list(str_last)):
		if a == b:
			common = common + a
		else:
			return common
	return None

def test_block_for_coverage(patterns, first_number, last_number):
	""" Helper function to allow testing for contiguous coverage of a block of numbers with a given set of patterns """
	patternList = []

	#convert patterns to regular expressions
	for pattern in patterns:
		pattern = re.sub(r'X', '\\d', pattern)
		pattern = re.sub(r'Z', '[1-9]', pattern)
		pattern = re.sub(r'N', '[2-9]', pattern)
		pattern = re.sub(r'^_', '', pattern)
		patternList.append(pattern)
	
	pre = re.compile("^("+"|".join(patternList)+")$")
	
	#test each number in range for 
	for x in range(int(first_number), int(last_number)+1):
		result = pre.match(str(x))
		if result is None:
			return False

	return True


from util import common_start
import math

def split_range(low, high):
	""" Take a range of telephone numbers in E.164 format between low and high and return
	the "safe" chunks for the pattern matching script to use 

	:param low: telephone number in E164 format
	:type low: integer
	:param high: telephoen number in E164 format
	:type high: integer
	:returns: list of integer tuples for pattern chunking
	:rtype: list
	"""

	from dialplan_exceptions import DialplanException

	#start by ensuring our arguments are integers
	low = int(low)
	high = int(high)

	#get string representations of numbers
	fn_string = str(low)
	ln_string = str(high)

	if len(fn_string) != len(ln_string):
		#the pattern to match is not valid
		raise DialplanException("First and last numbers are not of equal length")

	if low > high:
		raise DialplanException("Last number is smaller than the first number")

	if low == high:
		return [(low, high)]

	patterns = []

	#lets work out the number of significant digits
	pattern = common_start(low,high)
	digit_start = int(len(pattern))
	digits = len(str(low)) - len(pattern)
	strip_digits = int(digits*-1)
	remaining_fn = fn_string[strip_digits:]
	remaining_ln = ln_string[strip_digits:]

	remaining_fn_int = int(remaining_fn)
	remaining_ln_int = int(remaining_ln)

	last_of_first_range = int(math.ceil(float(remaining_fn_int/math.pow(10,digits-1)))*math.pow(10,digits-1))

	first_of_last_range = int(math.floor(float((remaining_ln_int+1)/math.pow(10,digits-1)))*math.pow(10,digits-1))
	
	if remaining_fn_int != last_of_first_range:
		low = pattern+str(remaining_fn_int).zfill(int(digits))
		high = pattern+str(last_of_first_range-1).zfill(int(digits))
		if strip_digits == 1:
			patterns.append((int(low), int(high)))
		else:
			patterns.extend(split_range(int(low), int(high)))

	if last_of_first_range != first_of_last_range:
		#we have a range in the middle
		low = pattern+str(last_of_first_range).zfill(int(digits))
		high = pattern+str(first_of_last_range-1).zfill(int(digits))
		patterns.append((int(low), int(high)))

	if (first_of_last_range-1) != remaining_ln_int:
		low = pattern+str(first_of_last_range).zfill(int(digits))
		high = pattern+str(remaining_ln_int).zfill(int(digits))
		if strip_digits == 1 or low == high:
			patterns.append((int(low), int(high)))
		else:
			patterns.extend(split_range(int(low), int(high)))

	
	return patterns


from setuptools import setup

setup(
	name="asterisk_dialplan",
	version="0.1.1",
	author="Andrew Yager",
	author_email="andrew@rwts.com.au",
	license="BSD",
	packages=['asterisk_dialplan'],
	description="Helpers to convert numbers to dialplan strings for use in Asterisk",
	classifiers=[
		"Development Status :: 4 - Beta",
		"Topic :: Communications :: Telephony",
		"Topic :: Software Development :: Libraries",
		"License :: OSI Approved :: BSD License",
		]
)
